package net.kazhik.android.tokyorunners;

import java.util.ArrayList;
import java.util.List;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.Point;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.GestureDetector;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.ItemizedOverlay;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.MyLocationOverlay;
import com.google.android.maps.Overlay;
import com.google.android.maps.OverlayItem;
import com.google.android.maps.Projection;

public class MapMode extends MapActivity implements
	LocationListener, GestureDetector.OnGestureListener {
	
	private LocationManager m_locMgr;
	private MapController m_controller;
	private GestureDetector m_gestureDetector;
	private List<Overlay> m_mapOverlays;
	private SplitPoints m_overlaySplitpoints;
	private String m_userName;
	private GeoPoint m_prevPoint = null;

	public class RunningRoute extends Overlay {

		private GeoPoint m_start;
		private GeoPoint m_end;

		public RunningRoute( GeoPoint ptStart, GeoPoint ptEnd) {
			m_start = ptStart;
			m_end = ptEnd;
		}

		@Override
		public void draw(Canvas canvas, MapView mapView, boolean shadow) {
			super.draw(canvas, mapView, shadow);

			if( !shadow ) {
				Paint paint = new Paint( Paint.ANTI_ALIAS_FLAG);
				paint.setStyle( Paint.Style.STROKE);
				paint.setAntiAlias( true);
				paint.setStrokeWidth( 3);
				paint.setColor(Color.YELLOW);

				Path path = new Path();
				Projection projection = mapView.getProjection();
				Point pxStart = projection.toPixels(m_start, null);
				Point pxEnd = projection.toPixels(m_end, null);
				path.moveTo( pxStart.x, pxStart.y);
				path.lineTo( pxEnd.x, pxEnd.y);

				canvas.drawPath(path, paint);
			}
		}
	}

	private class SplitPoints extends ItemizedOverlay<OverlayItem> {

		private ArrayList<OverlayItem> m_waypoints = new ArrayList<OverlayItem>();
		
		public SplitPoints(Drawable defaultMarker) {
			super(boundCenterBottom(defaultMarker));
		}

		public void addWayPoint(OverlayItem overlay) {
		    m_waypoints.add(overlay);
		    populate();
		}
		@Override
		protected OverlayItem createItem(int idx) {
			return m_waypoints.get(idx);
		}

		@Override
		public int size() {
			return m_waypoints.size();
		}
		
        @Override
        protected boolean onTap(int i) {
/*
        	if (m_waypoints.size() > 0) {
            	Toast.makeText(MapMode.this,
            			m_waypoints.get(i).getSnippet(), Toast.LENGTH_LONG).show();
        	}
*/
        	return(true);
        }
	}
	
	@Override
	protected boolean isRouteDisplayed() {
		return false;
	}

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.map);

        m_gestureDetector = new GestureDetector(this, this);


        SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
        
        String strGpsFreq = prefs.getString("gps_frequency", "0");
        int gps_freq =Integer.parseInt(strGpsFreq);
        
        m_locMgr = (LocationManager)getSystemService(Context.LOCATION_SERVICE);
    	m_locMgr.requestLocationUpdates(LocationManager.GPS_PROVIDER, gps_freq * 1000, 1, this);
//    	m_locMgr.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 0, this);


    	
        MapView m = (MapView)findViewById(R.id.mapview);
        m.setBuiltInZoomControls(true);

        m_controller = m.getController();
        m_controller.setZoom(16);

        m_mapOverlays = m.getOverlays();
        
        Drawable runner = getResources().getDrawable(R.drawable.runner);
        m_overlaySplitpoints = new SplitPoints(runner);
        
        MyLocationOverlay myLocationOverlay = new MyLocationOverlay(this, m);
        myLocationOverlay.enableMyLocation();

        m_mapOverlays.add(myLocationOverlay);
//        m_mapOverlays.add(m_overlayWaypoints);

        /*
		GeoPoint gp = new GeoPoint(35692134, 139815652);
		m_controller.animateTo(gp);
        */
        
        m_userName = prefs.getString("your_name", "Koike Kazuhiko");
    }
    @Override
    public void onStop() {
		m_locMgr.removeUpdates(this);
		super.onStop();
    }
    @Override
    public boolean onTouchEvent(MotionEvent event) {
		Log.d(this.getClass().getName(), "onTouchEvent()" );
    	
        if (m_gestureDetector.onTouchEvent(event)) {
        	return true;
        }
        return super.onTouchEvent(event);
    }
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        menu.add(Menu.NONE, Constants.MENU_STOPWATCH, Menu.NONE,
        		R.string.menu_stopwatch).setIcon(R.drawable.stopwatch);
        menu.add(Menu.NONE, Constants.MENU_SETTING, Menu.NONE,
        		R.string.menu_setting).setIcon(android.R.drawable.ic_menu_preferences);
        
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case Constants.MENU_STOPWATCH:
        	Intent intent = new Intent(this, TokyoRunners.class);
        	intent.setAction(Intent.ACTION_VIEW);
        	startActivity(intent);
        	
        	break;
        case Constants.MENU_SETTING:
        	intent = new Intent(this, Config.class);
        	intent.setAction(Intent.ACTION_VIEW);
        	startActivity(intent);
        	break;
        default:
            break;
        }
        return false;
    }       
	public void onLocationChanged(Location location) {
		Log.d(this.getClass().getName(),
				"onLocationChanged(): latitude: " + location.getLatitude() + 
				"; longitude: " + location.getLongitude() );
		
		GeoPoint gp = new GeoPoint((int)(location.getLatitude()*1E6),
				         (int)(location.getLongitude()*1E6));
		m_controller.animateTo(gp);
		
		
		/*
        SimpleDateFormat formatter = new SimpleDateFormat("HH:mm:ss");
        String snippet = "[" + m_userName + "]" + formatter.format(new Date());
        OverlayItem ovItem = new OverlayItem(gp, "", snippet);
        m_overlayWaypoints.addWayPoint(ovItem);
        */
		
		if (m_prevPoint != null) {
			RunningRoute route = new RunningRoute(m_prevPoint, gp);
			m_mapOverlays.add(route);
		}
		m_prevPoint = gp;
	}

	public void onProviderDisabled(String provider) {
	}

	public void onProviderEnabled(String provider) {
	}

	public void onStatusChanged(String provider, int status, Bundle extras) {
	}

	public boolean onDown(MotionEvent e) {
		return false;
	}

	public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX,
			float velocityY) {
		return false;
	}

	public void onLongPress(MotionEvent e) {
		Log.d(this.getClass().getName(), "onLongPress()" );
    	Intent intent = new Intent(this, TokyoRunners.class);
    	intent.setAction(Intent.ACTION_VIEW);
    	startActivity(intent);
		
	}

	public boolean onScroll(MotionEvent e1, MotionEvent e2, float distanceX,
			float distanceY) {
		return false;
	}

	public void onShowPress(MotionEvent e) {
		
	}

	public boolean onSingleTapUp(MotionEvent e) {
		return false;
	}
}
