package net.kazhik.android.tokyorunners;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.location.LocationProvider;
import android.os.Bundle;
import android.os.SystemClock;
import android.preference.PreferenceManager;
import android.text.format.DateUtils;
import android.util.Log;
import android.view.GestureDetector;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.WindowManager;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Chronometer;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.widget.TextView;

public class TokyoRunners extends Activity implements
	LocationListener, GestureDetector.OnGestureListener {
	
	private SharedPreferences m_prefs;
	private LocationManager m_locMgr;
	private long m_startTime;
	private Chronometer m_elapsedTime;
	private Chronometer m_lapTime;
	private boolean m_running = false;
	
	private boolean m_useGPS = false;
	private boolean m_autosplit = false;
	
	private GestureDetector m_gestureDetector;
	
	private RunningRecord m_runningRecord = new RunningRecord();
	
    class StartStopButton implements OnClickListener {
		public void onClick(View v) {
			if (m_running == true) {
				stop();
			} else {
				start();
			}
		}
    };	
    class ResetLapButton implements OnClickListener {
		public void onClick(View v) {
			if (m_running == true) {
				lap();
			} else {
				reset();
			}
		}
    };	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        m_gestureDetector = new GestureDetector(this, this);
        
        if (m_gestureDetector.isLongpressEnabled() == false) {
    		Log.d(this.getClass().getName(), "Longpress disabled");
        }
        setContentView(R.layout.stopwatch);

        m_prefs = PreferenceManager.getDefaultSharedPreferences(this);
        
        m_useGPS = m_prefs.getBoolean("use_gps", true);
        
        if (m_useGPS) {
        	m_locMgr = (LocationManager)getSystemService(Context.LOCATION_SERVICE);
        	
        	m_autosplit = m_prefs.getBoolean("auto_split", false);
            if (m_autosplit) {
            	int interval = m_prefs.getInt("split_interval", 5);
            }

        }
        
        m_elapsedTime = (Chronometer)findViewById(R.id.elapsed_time);
        m_lapTime = (Chronometer)findViewById(R.id.lap_time);
        
        Button btnStartStop = (Button)findViewById(R.id.button_start_stop);
	    btnStartStop.setOnClickListener(new StartStopButton());
	    
        Button btnResetLap = (Button)findViewById(R.id.button_reset_lap);
	    btnResetLap.setOnClickListener(new ResetLapButton());
	    
		ListView lvLaptime = (ListView)findViewById(R.id.lap_history);
        ArrayAdapter<String> arrayAdapter
        	= new ArrayAdapter<String>(this, R.layout.laptime);
        lvLaptime.setAdapter(arrayAdapter);

        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);        
        
    }
    
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        switch (requestCode) {
        case Constants.REQUEST_CODE_SETTINGS:
            m_useGPS = m_prefs.getBoolean("use_gps", true);
            break;
        case Constants.REQUEST_CODE_HISTORY:
        	break;
        }
    }
    
    
    @Override
    public void onDestroy() {
        getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);    
    	
    }
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        menu.add(Menu.NONE, Constants.MENU_SAVE, Menu.NONE, R.string.menu_save).setIcon(android.R.drawable.ic_menu_save);
        menu.add(Menu.NONE, Constants.MENU_HISTORY, Menu.NONE, R.string.menu_history).setIcon(R.drawable.history2);
        menu.add(Menu.NONE, Constants.MENU_MAP, Menu.NONE, R.string.menu_map).setIcon(android.R.drawable.ic_menu_mapmode);
        menu.add(Menu.NONE, Constants.MENU_SETTING, Menu.NONE, R.string.menu_setting).setIcon(android.R.drawable.ic_menu_preferences);
        
        return super.onCreateOptionsMenu(menu);
    }
    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
    	menu.findItem(Constants.MENU_MAP).setEnabled(m_useGPS);
        
        return super.onPrepareOptionsMenu(menu);
    }
    
    @Override
    public Dialog onCreateDialog (int id) {
    	
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	switch (id) {
    	case Constants.DIALOG_SAVE_ID:
    		builder.setTitle(R.string.dialog_save_title);
    		builder.setMessage(R.string.dialog_save_message);
    		builder.setPositiveButton(R.string.dialog_yes, new DialogInterface.OnClickListener() {
				@Override
				public void onClick(DialogInterface dialog, int which) {
					saveRecord();
					// TODO Auto-generated method stub
				}
			});
    		builder.setNegativeButton(R.string.dialog_no, new DialogInterface.OnClickListener() {
				@Override
				public void onClick(DialogInterface dialog, int which) {
					dialog.cancel();
					
				}
			});

    		break;
		default:
			break;
    	}
    	
    	AlertDialog alertDialog = builder.create();
    	return alertDialog;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
    	Intent intent;
        switch (item.getItemId()) {
        case Constants.MENU_SAVE:
        	showDialog(Constants.DIALOG_SAVE_ID);
        	break;
        case Constants.MENU_HISTORY:
        	intent = new Intent(this, RunningHistory.class);
        	intent.setAction(Intent.ACTION_VIEW);
        	intent.putExtra("files", fileList());
        	startActivityForResult(intent, Constants.REQUEST_CODE_HISTORY);
        	break;
        case Constants.MENU_MAP:
        	intent = new Intent(this, MapMode.class);
        	intent.setAction(Intent.ACTION_VIEW);
        	startActivity(intent);
        	break;
        case Constants.MENU_SETTING:
        	intent = new Intent(this, Config.class);
        	intent.setAction(Intent.ACTION_VIEW);
        	startActivityForResult(intent, Constants.REQUEST_CODE_SETTINGS);
        	break;
        default:
            break;
        }
        return false;
    }       
	public void onLocationChanged(Location location) {
		Log.d(this.getClass().getName(),
				"onLocationChanged(): latitude: " + location.getLatitude() + 
				"; longitude: " + location.getLongitude() );
		
		m_runningRecord.addRecord(new Date(), location);
		
        float distance_km = m_runningRecord.getDistance();
        TextView distanceView = (TextView)findViewById(R.id.distance);
        distanceView.setText(Float.toString(distance_km));
        
		
	}

	public void onProviderDisabled(String provider) {
		
	}

	public void onProviderEnabled(String provider) {
		
	}

	public void onStatusChanged(String provider, int status, Bundle extras) {
		String statusStr = "";
		switch (status) {
		case LocationProvider.AVAILABLE:
			statusStr = "Available";
			break;
		case LocationProvider.OUT_OF_SERVICE:
			statusStr = "Out of service";
			break;
		case LocationProvider.TEMPORARILY_UNAVAILABLE:
			statusStr = "Temporarily unavailable";
			break;
		default:
			statusStr = "Unknown status";
			break;
		}
		Log.d(this.getClass().getName(), "onStatusChanged(): " + statusStr);
		
	}
	private void start() {
		m_startTime = System.currentTimeMillis();
		long startTime = SystemClock.elapsedRealtime();
        m_elapsedTime.setBase(startTime);
        m_lapTime.setBase(startTime);
        m_elapsedTime.start();
        m_lapTime.start();
        
        if (m_useGPS) {
            String strGpsFreq = m_prefs.getString("gps_frequency", "0");
            int gps_freq =Integer.parseInt(strGpsFreq);
        	m_locMgr.requestLocationUpdates(LocationManager.GPS_PROVIDER, gps_freq * 1000, 1, this);
        	
        }

    	Button startStopButton = (Button)findViewById(R.id.button_start_stop);
        Button resetLapButton = (Button)findViewById(R.id.button_reset_lap);
		startStopButton.setText(R.string.button_stop);
		resetLapButton.setText(R.string.button_lap);
		m_running = true;
	}
	private void stop() {
		m_elapsedTime.stop();
		m_lapTime.stop();
		
        if (m_useGPS) {
    		m_locMgr.removeUpdates(this);
        }

        Button startStopButton = (Button)findViewById(R.id.button_start_stop);
        Button resetLapButton = (Button)findViewById(R.id.button_reset_lap);
		startStopButton.setText(R.string.button_start);
		resetLapButton.setText(R.string.button_reset);
		m_running = false;
		
	}
	private void reset() {
		long elapsedRealTime = SystemClock.elapsedRealtime();
		m_elapsedTime.setBase(elapsedRealTime);
        m_lapTime.setBase(elapsedRealTime);
		
		ListView listLaptime = (ListView)findViewById(R.id.lap_history);
		ArrayAdapter<String> lapHistory = (ArrayAdapter<String>)listLaptime.getAdapter();
		lapHistory.clear();
		
	}
	private void lap() {
		long elapsedRealTime = SystemClock.elapsedRealtime();
		long lapTimeMillis = elapsedRealTime - m_lapTime.getBase();
		
		ListView listLaptime = (ListView)findViewById(R.id.lap_history);
		
		
		
		ArrayAdapter<String> lapHistory = (ArrayAdapter<String>)listLaptime.getAdapter();
		
		int historyCount = lapHistory.getCount() + 1;
		
		String strLaptime = "laptime " + historyCount + ": "
			+ DateUtils.formatElapsedTime(lapTimeMillis / 1000);
		
		lapHistory.insert(strLaptime, 0);
		
		m_lapTime.setBase(elapsedRealTime);
		
	}
	
	private void saveRecord() {
		FileOutputStream outFile = null;
		OutputStreamWriter stWriter = null;
		try {
		    SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd_HH_mm_ss");	
			outFile = openFileOutput(formatter.format(new Date(m_startTime)) + ".xml", MODE_PRIVATE);
			stWriter = new OutputStreamWriter(outFile);
			m_runningRecord.save(stWriter);
		} catch (IOException e) {
		} finally {
			try {
				if (stWriter != null) {
					stWriter.close();
				}
				if (outFile != null) {
					outFile.close();
				}
			} catch (IOException e) {
			}
		}

	}
	
    @Override
    public boolean onTouchEvent(MotionEvent event) {
		Log.d(this.getClass().getName(), "onTouchEvent():" );
        if (m_gestureDetector.onTouchEvent(event)) {
        	return true;
        }
        return super.onTouchEvent(event);
    }

	
	public boolean onDown(MotionEvent arg0) {
		Log.d(this.getClass().getName(), "onDown():" );
		return false;
	}
	public boolean onFling(MotionEvent arg0, MotionEvent arg1, float arg2,
			float arg3) {
		Log.d(this.getClass().getName(), "onFling():" );
		return false;
	}
	public void onLongPress(MotionEvent arg0) {
		Log.d(this.getClass().getName(), "onLongPress():" );
		
    	Intent intent = new Intent(TokyoRunners.this, MapMode.class);
    	intent.setAction(Intent.ACTION_VIEW);
    	startActivity(intent);
		
	}
	public boolean onScroll(MotionEvent arg0, MotionEvent arg1, float arg2,
			float arg3) {
		Log.d(this.getClass().getName(), "onScroll():" );
		return false;
	}
	public void onShowPress(MotionEvent arg0) {
		Log.d(this.getClass().getName(), "onShowPress():" );
	}
	public boolean onSingleTapUp(MotionEvent arg0) {
		Log.d(this.getClass().getName(), "onSingleTapUp():" );
		return false;
	}
}